


#import "Spline.h"

@implementation Spline

- initWithContentsOfFile:(NSString *)fileName;
{
    NSDictionary *readDictionary;
    NSMutableDictionary *dictionary=[NSMutableDictionary dictionary];
    NSEnumerator *enumerator;
    id keyObject;
    int i;
    double s,t;
    
    [super init];
    if( ! (readDictionary = [NSDictionary dictionaryWithContentsOfFile:fileName]) )
    {
        NSLog(@"Can't create dictionary from File: %@",fileName);
        [self dealloc];
        return nil;
    }

    count = [readDictionary count];
    xvalues = (double *)malloc(sizeof(double) * count);
    yvalues = (double *)malloc(sizeof(double) * count);

    bvalues = (double *)malloc(sizeof(double) * count);
    cvalues = (double *)malloc(sizeof(double) * count);
    dvalues = (double *)malloc(sizeof(double) * count);

    enumerator = [readDictionary keyEnumerator];
    while( keyObject = [enumerator nextObject] )
    {
        [dictionary setObject:[NSNumber numberWithDouble:[keyObject doubleValue]]
                      forKey:[NSNumber numberWithDouble:[[readDictionary objectForKey:keyObject] doubleValue]]];
    }

    enumerator = [[dictionary keysSortedByValueUsingSelector:@selector(compare:)] objectEnumerator];
    i = 0;
    while( keyObject = [enumerator nextObject] )
    {
        xvalues[i] = [[dictionary objectForKey:keyObject] doubleValue];
        yvalues[i] = [keyObject doubleValue];
        //NSLog(@"X,Y:%10.5f %10.5f",xvalues[i],yvalues[i]);
        i++;
    }


    
    for( i=1; i<count; i++)
    {
        bvalues[i] =  xvalues[i]-xvalues[i-1];
    }
    for( i=1; i<count-1; i++)
    {
        dvalues[i] = 2*(bvalues[i] + bvalues[i+1]);
        cvalues[i] = 6*(-(yvalues[i]-yvalues[i-1])/bvalues[i]+(yvalues[i+1]-yvalues[i])/bvalues[i+1]);
    }
    t = cvalues[1];
    for( i=2; i<count-1; i++)
    {
        s = dvalues[i-1];
        dvalues[i] = dvalues[i]-bvalues[i]*bvalues[i]/s;
        t = cvalues[i]-bvalues[i]*t/s;
        cvalues[i]=t;
    }
    cvalues[count-2]=cvalues[count-2]/dvalues[count-2];
    for( i=count-3; i>0; i-- )
    {
        cvalues[i]=(cvalues[i]-cvalues[i+1]*bvalues[i+1])/dvalues[i];
    }
    cvalues[0]=0.0;
    cvalues[count-1]=0.0;

    for( i=0; i<count-1; i++)
    {
        dvalues[i] = (cvalues[i+1]-cvalues[i])/(6*bvalues[i+1]);
    }
    for( i=0; i<count-1; i++)
    {
        bvalues[i] = (yvalues[i+1]-yvalues[i])/bvalues[i+1]-cvalues[i+1]*bvalues[i+1]/6-cvalues[i]*bvalues[i+1]/3;
    }
    for( i=0; i<count-1; i++)
    {
        cvalues[i] = cvalues[i]/2;
    }
    
    return self;
}

- (void) dealloc
{
    free(xvalues);
    free(yvalues);
    free(bvalues);
    free(cvalues);
    free(dvalues);
    [super dealloc];
}

- (NSNumber *)valueAt:(NSNumber *)xPosition;
{
    double xposition;
    double yposition,k;
    int i;

    xposition=[xPosition doubleValue];


    if(xposition < xvalues[count-1])
    {
        i=-1;
        do
        {
            i++;
        }
        while( (xposition >= xvalues[i+1]));
    }
    else
    {
        i=count-1;
    }

    k = xposition-xvalues[i];
    yposition = yvalues[i]+k*(bvalues[i]+k*(cvalues[i]+k*dvalues[i]));

    //NSLog(@"i,b,c,d,x,y:%3d %8.5f %8.5f %8.5f %5.2f %8.4f",i,bvalues[i],cvalues[i],dvalues[i],xposition,yposition);

    return [NSNumber numberWithDouble:yposition];
}
@end



