
#import "SimulationManager.h"


@implementation SimulationManager



- actualIntegrator;
{
	return actualIntegrator;
}

- openSimulation:sender;
{
    NSArray *fileFypes = [NSArray arrayWithObject:SIMULATOR_TYPE];
    NSOpenPanel *openPanel = [NSOpenPanel openPanel];

    [openPanel setTitle:@"open Simulation"];	
    if(! [openPanel runModalForTypes:fileFypes] )
        return nil;

    return [self openSimulationWithFilename:[openPanel filename]];
}



- openSimulationWithFilename:(NSString *)fileName;
{
    NSBundle *myBundle;
    NSString *newClassname;
    NSString *nibFilename;


    if( ! [NSBundle loadNibNamed:DOCUMENT_NIB owner:self])
    {
        NSRunAlertPanel(@"Dynamic loading", @"Can't load DocumentView %s", @"OK", nil, nil, DOCUMENT_NIB);
        return nil;
    }

    if(! ( myBundle = [[NSBundle alloc] initWithPath:fileName]))
    {
        NSRunAlertPanel(@"Dynamic loading", @"Can't load bundle %s", @"OK", nil, nil, fileName);
        return nil;
    }
	
    newClassname = [[fileName lastPathComponent] stringByDeletingPathExtension];
    [simulationViewWindow setTitle:[NSString stringWithFormat:@"Simulation - %@",newClassname]];
    [simulationViewWindow setFrameAutosaveName:[NSString stringWithFormat:@"Simulation - %@",newClassname]];
    [simulationParameterWindow setTitle:[NSString stringWithFormat:@"Parameters - %@",newClassname]];
    [simulationParameterWindow setFrameAutosaveName:[NSString stringWithFormat:@"Parameters - %@",newClassname]];

    if(!( [myBundle classNamed:newClassname]))
    {
        NSRunAlertPanel(@"Dynamic loading", @"Can't load Class named %s", @"OK", nil, nil, newClassname);
        return nil;
    }

    if(nil == (nibFilename = [myBundle pathForResource:newClassname ofType:@"nib"]))
    {
        NSRunAlertPanel(@"Dynamic loading", @"Can't find %s nibfile in Bundle named ", @"OK", nil, nil, newClassname);
        return nil;
    }
    if( ![NSBundle loadNibFile:nibFilename
             externalNameTable:[NSDictionary dictionaryWithObjectsAndKeys:loadingSimulator, @"NSOwner", nil]
                      withZone:[loadingSimulator zone]])
    {
        NSRunAlertPanel(@"Dynamic loading", @"Can't load Nib %s", @"OK", nil, nil, fileName);
        return nil;
    }
    
    return self;
}


- openIntegrator:sender;
{
    NSArray *fileTypes = [NSArray arrayWithObject:INTEGRATOR_TYPE];
    id openPanel = [NSOpenPanel openPanel];

    [openPanel setTitle:@"open Integrator"];	
    if(! [openPanel runModalForTypes:fileTypes] )
        return nil;
    return [self openIntegratorWithFilename:[openPanel filename]];
}


- openIntegratorWithFilename:(NSString *)fileName
{
    NSBundle		*myBundle;
    NSString		*newClassname;

    if(! ( myBundle = [[NSBundle alloc] initWithPath:fileName]))
    {
        NSRunAlertPanel(@"Dynamic loading", @"Can't load bundle %s", @"OK", nil, nil, fileName);
        return nil;
    }

    newClassname = [[fileName lastPathComponent] stringByDeletingPathExtension];
    if(!( [myBundle classNamed:newClassname]))
    {
        NSRunAlertPanel(@"Dynamic loading", @"Can't load Class named %s", @"OK", nil, nil, newClassname);
        return nil;
    }

    actualIntegrator = [[[myBundle classNamed:newClassname] alloc] init];
    return self;
}



- simulationDidFinish:sender;
{
	return self;
}


/*****
	Methods cause we are Application delegate
*****/


- (int)application:sender openFile:(NSString *)fileName
{
    NSString	*fileType=[fileName pathExtension];

    if(NSOrderedSame == [fileType compare:SIMULATOR_TYPE])
    {
        if([self openSimulationWithFilename:fileName])
            return YES;
        return NO;
    }
    if(NSOrderedSame == [fileType compare:INTEGRATOR_TYPE])
    {
        if([self openIntegratorWithFilename:fileName])
            return YES;
        return NO;
    }
    return NO;	
}



@end
