
#import "SimulationView.h"
#import "SimulationManager.h"
#import "SimulationProtocols.h"




@implementation SimulationView

- (void)drawRect:(NSRect)rect;
{
    if(maximumTimeDisplayed < [timeMax doubleValue])
    {
        maximumTimeDisplayed = [timeMax doubleValue];
    }
    [self setBoundsSize:NSMakeSize([timeMax doubleValue], [yAxis doubleValue]*2)];
    [self setBoundsOrigin:NSMakePoint(maximumTimeDisplayed - [timeMax doubleValue], - [yAxis doubleValue])];

    PSsetgray(1.0);
    PSrectfill([self bounds].origin.x,[self bounds].origin.y,[self bounds].size.width,[self bounds].size.height);
    PSsetlinewidth(0.0);
    PSsetgray(0.0);
    PSmoveto(0.0,0.0);
    PSlineto(maximumTimeDisplayed,0.0);
    PSstroke();
}

- resetSimulation:sender;
{
    maximumTimeDisplayed = [timeMax doubleValue];
    [simulator resetSimulator];
    actualTime = 0.0;
    [self display];
    return self;
}

- simulationButtonPressed:sender;
{
    if([simulationButton state])			// cause it will change the state
    {
        myTimedEntry = [[NSTimer scheduledTimerWithTimeInterval:[simulationSpeed doubleValue]
                                                         target:self selector:@selector( simulateOneStep:)
                                                       userInfo:self repeats:YES] retain];
    }
    else
    {
        [myTimedEntry invalidate];
        [myTimedEntry release];
    }
    return self;
}



- (void)simulateOneStep:timer
{
    NSEnumerator *enumerator;
    NSArray *points;
    NSArray *object;

    actualTime+=[timeDelta doubleValue];

    if(actualTime >= [self bounds].origin.x + [self bounds].size.width)
    {
        NSRect	newbounds=[self bounds];
        NSSize size;
        size.width =  - (newbounds.size.width/20.0);
        size.height = 0;

        [self scrollRect:newbounds by:size];	
        [self setBoundsOrigin:NSMakePoint([self bounds].origin.x-size.width, [self bounds].origin.y)];
        [self lockFocus];
        PSsetgray(1.0);
        PSrectfill([self bounds].origin.x+[self bounds].size.width+size.width,
                   [self bounds].origin.y,-size.width, [self bounds].size.height);
        PSstroke();
        PSsetlinewidth(0.0);
        PSsetgray(0.0);
        PSmoveto([self bounds].origin.x+[self bounds].size.width+size.width,0.0);
        PSrlineto([self bounds].size.width,0.0);
        PSstroke();
        maximumTimeDisplayed = [self bounds].origin.x + [self bounds].size.width;
        [self unlockFocus];
    }

    points = [simulator simulateOneStepWithIntegrator:[simulationManager actualIntegrator]
                                                delta:[NSNumber numberWithDouble:[timeDelta doubleValue]]
                                              epsilon:[NSNumber numberWithDouble:[simulationEpsilon doubleValue]]];
    enumerator = [points objectEnumerator];

    [self lockFocus];
    PSsetlinewidth(0.0);

    while( object = [enumerator nextObject])
    {
        float	height = [[object objectAtIndex:0] floatValue];
        if( (height < [self bounds].origin.y+[self bounds].size.height) && (height > [self bounds].origin.y) )
        {
            [[object objectAtIndex:1] set];
            PSmoveto(actualTime,[[object objectAtIndex:0] doubleValue]);
            PSlineto(actualTime,[[object objectAtIndex:0] doubleValue]);
            PSstroke();
        }
    }
    [self unlockFocus];
}


- (void)windowDidMiniaturize:(NSNotification *)notification;
{
	return;
}
- (void)windowDidDeminiaturize:(NSNotification *)notification;
{
	return;
}
- (BOOL)windowShouldClose:(id)sender;
{
	return YES;
}
@end
